/**
* @fileOverview Plugin to style the Html TileView elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejTile
    * @requires jQuery
    * @requires ej.core
    * @requires ej.unobtrusive
    * @requires ej.data
    * @requires ej.touch
    * @requires ej.tilebase
    * @classdesc Custom Design for Html Tile control.
    * @example 
    * &lt;div id="tile" &gt;&lt;/div&gt;<br>
    * &lt;script&gt; 
    * // Create Tile control 
    * $("#tile").ejTile({ imageUrl: "people.png" }); 
    * &lt;/script&gt;
    */

    // ejTile is the plugin name 
    // "ej.Tile" is "namespace.className" will hold functions and properties

    ej.widget("ejTile", "ej.Tile", {
        _setFirst: true,
        validTags: ["div"],
        _rootCSS: "e-tile",
        defaults: {
            /**    
            * Event triggers when the mousedown happens in the tile
            * @event
            * @name ejTile#mouseDown 	
            * @param {Object} argument Event parameters from tile     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {boolean} argument.model returns the tile model
            * @param {boolean} argument.type returns the name of the event
            * @param {boolean} argument.text returns the current tile text 
            * @example 
            * // Define mouseDown event on initialization. 
            * // To set mouseDown event API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejTile({ imageUrl: "people.png", 
            * mouseDown: function (args) { 
            * //handle the event 
            * }
            * }); 
            * &lt;/script&gt;
            * @memberof ejTile
            * @instance
            */
            mouseDown: null,
            /**    
            * Event triggers when the mouseup happens in the tile
            * @event
            * @name ejTile#mouseUp 	
            * @param {Object} argument Event parameters from tile     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {boolean} argument.model returns the tile model
            * @param {boolean} argument.type returns the name of the event
            * @param {boolean} argument.text returns the current tile text         
            * @example 
            * // Define mouseUp event on initialization. 
            * // To set mouseUp event API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejTile({ imageUrl: "people.png", 
            * mouseUp: function (args) { 
            * //handle the event 
            * }
            * }); 
            * &lt;/script&gt;
            * @memberof ejTile
            * @instance
            */
            mouseUp: null
        },
        //Constructor function
        _init: function () {
            this._orgEle = this.element.clone();
            this._renderEJControl();
        },
        //To render the Tile control
        _renderEJControl: function () {
            this.element.addClass("e-tile-web");
            this._prefix = "e-";
            this._tileRender();
            this._mainTile.append(this._layer);
            this._wireEvents();
        },
        //Section for handling the mousedown event
        _onMouseDownHandler: function (evt) {
            var item = this.element;
            item.addClass(ej._getSkewClass(item, evt.pageX, evt.pageY).replace('e-m-', 'e-'));
            if (this.model.mouseDown)
                this._trigger("mouseDown", { text: item.find(".e-tile-text").html(), index: $(evt.target).index() });
            ej.listenTouchEvent(this.element, ej.moveEvent(), this._onMoveDelegate, false, this);
            ej.listenTouchEvent(this.element, ej.endEvent(), this._onEndDelegate, false, this);
            this._isMoved = false;
        },
        //Section for handling the mousemove event
        _onMouseMoveHandler: function (evt) {
            this._isMoved = true;
            this._removeActiveClass();
        },
        //Section for handling the mouseup event
        _onMouseUpHandler: function (evt) {
            if (!this._isMoved && this.model.mouseUp)
                this._trigger("mouseUp", { text: this.element.find(".e-tile-text").html(), index: $(evt.target).index() });
            this._removeActiveClass();
            this._isMoved = false;
            ej.listenTouchEvent(this.element, ej.moveEvent(), this._onMoveDelegate, true, this);
            ej.listenTouchEvent(this.element, ej.endEvent(), this._onEndDelegate, true, this);
        },
        //To refresh the control
        _refresh: function () {
            this._destroy();
            this.element.addClass("e-tile");
            this._renderEJControl();
        }
    });
    /**
    * Enum for Tile ImagePosition
    * @enum {string}
    * @global
    */
    ej.Tile.ImagePosition = {
        /**
        Enum for center position of tile image*/
        Center: "center",
        /**
        Enum for top position of tile image*/
        Top: "top",
        /**
        Enum for bottom position of tile image*/
        Bottom: "bottom",
        /**
        Enum for right position of tile image*/
        Right: "right",
        /**
        Enum for left position of tile image*/
        Left: "left",
        /**
        Enum for topleft position of tile image*/
        TopLeft: "topleft",
        /**
        Enum for topright position of tile image*/
        TopRight: "topright",
        /**
        Enum for bottomright position of tile image*/
        BottomRight: "bottomright",
        /**
        Enum for bottomleft position of tile image*/
        BottomLeft: "bottomleft",
        /**
        Enum for fill position of tile image*/
        Fill: "fill"
    },
    /**
    * Enum for Tile TextAlignment
    * @enum {string}
    * @global
    */
    ej.Tile.TextAlignment = {
        /**
        Enum for normal alignment of tile text*/
        Normal: "normal",
        /**
        Enum for left alignment of tile text*/
        Left: "left",
        /**
        Enum for right alignment of tile text*/
        Right: "right",
        /**
        Enum for center alignment of tile text*/
        Center: "center"
    },
    ej.Tile.TextPosition = {
        /**
        Enum for inner position of tile text*/
        Inner: "inner",
        /**
        Enum for outer position of tile text*/
        Outer: "outer"
    },
    /**
    * Enum for Tile Size
    * @enum {string}
    * @global
    */
    ej.Tile.TileSize = {
        /**
        Enum for medium size of tile*/
        Medium: "medium",
        /**
        Enum for small size of tile*/
        Small: "small",
        /**
        Enum for large size of tile*/
        Large: "large",
        /**
        Enum for wide size of tile*/
        Wide: "wide"
    },
    /**
    * Enum for LiveTileType
    * @enum {string}
    * @global
    */
    ej.Tile.LiveTileType = {
        /**
        Enum for flip type of livetile*/
        Flip: "flip",
        /**
        Enum for slide type of livetile*/
        Slide: "slide",
        /**
        Enum for carousel type of livetile*/
        Carousel: "carousel"
    }
    $.extend(true, ej.Tile.prototype, ej.TileBase.prototype);
})(jQuery, Syncfusion);